//////////////////////////////////////////////////////////////
/*                  Speech Button & Pop up                  */
//////////////////////////////////////////////////////////////
//Movement of button derived from the below codepen.
//Copyright (c) 2019 by Simone Bernabè (https://codepen.io/simoberny/pen/ObyejY)

if ($(".j-bubble").length) {
    $(".j-bubble").draggable();
}


var isMoving = false;
var isdragging = false;
var speechMode = false;

//When the user is speaking do this.
function speaking() {
    $("#loader").removeClass("hidden");
}

//When the user stops speaking do this.
function stopSpeaking() {
    $("#loader").addClass("hidden");
}

//Function to close the pop up
function closePop() {
    speechMode = false;

    $("#j-wrap").fadeOut("fast", function () {
        $("#j-wrap").addClass("hidden");
    });

    $("body").removeClass("no-flow");
    $(".j-bubble").draggable('enable');
    resetResponseMsg();
    stopListening();
    stopSpeech();

}

//Function to open the pop up
function openPop() {
    speechMode = true;
    $("#j-wrap").fadeIn("fast", function () {
        $("#j-wrap").removeClass("hidden");
    });
    $("body").addClass("no-flow");
    $(".j-bubble").draggable('disable');
    playPop();
}

function playPop() {

    var audio = new Audio('/jinn/sound/soundeffect.mp3'); //Target for the sound file.
    audio.play();

}

//Clicking the speech button
$(".j-bubble").on("click", function () {
    if (!isdragging) { //If the button is not being dragged

        stopSpeech();

        if (speechMode) {
            closePop();

        } else {
            openPop();
        }
    }
});

$("#jClose").on("click", function () {

    stopSpeech();
    closePop();

});

$("#jAgain").on("click", function () {

    if (speechMode == true) {

        stopSpeech();
        resetResponseMsg();
        stopListening();
        speechMode = false;

    }

    $("#loader").removeClass("hidden");
    speechMode = true;
    playPop();
    recognition.start();

});


$(".j-bubble").mousedown(function () {
    isdragging = false;
});

$(".j-bubble").mousemove(function () {
    isdragging = true;
    $(this).css("transition", "all 0s");
});

$(".j-bubble").mouseup(function (e) {
    e.preventDefault();
    var lastY = window.event.clientY;
    var lastX = window.event.clientX;
    var swidth = $(window).width();

    if (isdragging) {

        if (speechMode) {
            speechMode = false;
            closePop();
        }

        if (lastX > (swidth / 2)) {
            $(this).css("top", lastY).css("left", (swidth - 55) + "px").css("transition", "all 0.4s");
        } else {
            $(this).css("top", lastY).css("left", "-5px").css("transition", "all 0.4s");
        }
    }
});

//////////////////////////////////////////////////////////////
/*                     Speech to Text                       */
//////////////////////////////////////////////////////////////

var message = document.querySelector('#j-results');
var title = document.querySelector('#j-instr');
var response = document.querySelector('#response');

var SpeechRecognition = SpeechRecognition || webkitSpeechRecognition;
var SpeechGrammarList = SpeechGrammarList || webkitSpeechGrammarList;

var grammer = '#JSGF V1.0;';
var lastMessage;

var recognition = new SpeechRecognition();
var speechRecognitionGrammarList = new SpeechGrammarList();
speechRecognitionGrammarList.addFromString(grammer, 1);

//? recognition.grammers
recognition.grammers = speechRecognitionGrammarList;
recognition.lang = 'en-US';
recognition.interimResults = true;

recognition.onresult = function (event) {

    var interim_transcript = '';

    if (typeof (event.results) == 'undefined') {
        recognition.onend = null;
        recognition.stop();
        upgrade();
        return;
    }

    for (var i = event.resultIndex; i < event.results.length; ++i) {
        if (event.results[i].isFinal) {
            lastMessage += event.results[i][0].transcript;
        } else {
            interim_transcript += event.results[i][0].transcript;
            message.textContent = interim_transcript;
            lastMessage = event.results[i][0].transcript;
        }
    }

};

recognition.onspeechend = function (event) {

    recognition.stop();
    decipherCmd(lastMessage);
    $("#loader").addClass("hidden");
}

recognition.onerror = function (event) {

    $("#loader").addClass("hidden");
    message.textContent = 'Error occurred in recognition: ' + event.error;
    lastMessage = 'Error occurred in recognition: ' + event.error;

}

recognition.stop = function () {

    $("#loader").addClass("hidden");

}

if ($(".j-bubble").length) { // if bubble exists.

    $(".j-bubble").click(function () { //add event listener to the button
        if (speechMode) {
            $("#loader").removeClass("hidden");
            recognition.start();

        } else {
            $("#loader").addClass("hidden");
            recognition.stop();
            resetResponseMsg();
        }
    });

}

function tryMe(cmd) {
    console.log(cmd);
}

function resetResponseMsg() {
    message.textContent = '';
    response.textContent = '';
}

function decipherCmd(cmd) {

    if (cmd !== '') {
        var command = message.textContent;
        command = command.toLowerCase()
        check(command);
    }

}

//////////////////////////////////////////////////////////////
/*                     Text to Speech                       */
//////////////////////////////////////////////////////////////

const synth = window.speechSynthesis;
const voiceSelect = 1;
const rate = 1;
const rateValue = 1;
const pitch = 1;
const pitchValue = 1;

// Speak
const speak = (cmd) => {

    //Check if speaking
    if (synth.speaking) {
        console.error('Already speaking...');
        return;
    }

    if (cmd !== '') {

        // Get speak text
        const speakText = new SpeechSynthesisUtterance(cmd);

        // Speak End
        speakText.onend = e => {
            console.log('Done Speaking...');
        }

        // Speak error
        speakText.onerror = e => {
            console.error('Something went wrong');
        }

        // Set pitch and rate
        speakText.rate = rate;
        speakText.pitch = pitch;

        // Speak
        synth.speak(speakText);
        response.textContent = cmd;
    }
};

function stopSpeech() {
    if (synth.speaking) {
        synth.cancel();
    }
}

function stopListening() {
    recognition.stop();
}

////////////////////////////////////////////////////////////////
/* Searching through JSON, acting and responding to the user. */
////////////////////////////////////////////////////////////////

//This examples is able to search JSON using javascript.
//data being gathered from local JSON file.
var commands = ($.getJSON("/jinn/commands.json", function (data) {
    commands = data;
    return commands;

}));

function check(e) {

    console.log('Transcript: ' + e);
    var script = e;
    var cmdNum = "null";
    var keyType = false;

    //This following function is used to check all of the fixed cmds.
    for (var a = 0; a < commands.fixedCmd.length; a++) {
        for (var b = 0; b < commands.fixedCmd[a].cmd.length; b++) {
            if (script.includes(commands.fixedCmd[a].cmd[b])) {
                var cmdNum = a; //This number is equal to the number in which value in the array.

                // The following code checks to see if the "key" matches the command
                //if no key exists, make the keyType true and finalise the function, ending this function
                if (commands.fixedCmd[cmdNum].key.length == 0) {
                    keyType = true;
                    finalise(cmdNum, keyType);
                    return;
                } else { //if a key exists, for each key check to see if it matches what has been spoken
                    // if it does exist make the keytype true, finalise and end the function.
                    for (var c = 0; c < commands.fixedCmd[cmdNum].key.length; c++) {

                        if (script.includes(commands.fixedCmd[cmdNum].key[c])) {
                            keyType = true;
                            console.log(script)
                            finalise(cmdNum, keyType);
                            return;
                        }

                    }
                }
            }
        }
    }

    // if no command and keytype has been found, the key type will remain false. Finalise the code without the parameters, causing no action to take place.
    if (keyType != true) {
        finalise();
    }
}

//How to utilise the check function.
//check(inputSpokenString);

//This function will determine if an action is to be taken, and if so which one.
function finalise(e, i) {
    var cmdNum = e;
    var keyType = i;

    console.log(cmdNum);
    console.log(keyType);

    if (cmdNum != null && keyType == true) {
        //console.log("Action found");
        //commands.fixedCmd[cmdNum].action;
        console.log("Action: " + commands.fixedCmd[cmdNum].action);

        //This is the combination of the action and response stored in the JSON making it one function
        var combine = commands.fixedCmd[cmdNum].action + commands.fixedCmd[cmdNum].resp;
        var f = new Function(combine);
        f();
        console.log("Respond: " + commands.fixedCmd[cmdNum].resp);
        //response.textContent = "";
    } else {
        //console.log(cmdNum);
        console.log("No Action found");
        speak("Sorry, I don't know what you mean, please try again.");
        response.textContent = "Say 'help' for further assistance...";

    }
}




//////////////////////////////////////////////////////////////////
/*                  Adding Commands JavaScript                  */
//////////////////////////////////////////////////////////////////

//toggles visibility of the help boxxes
function toggleHelp() {
    $(".helpText").toggleClass("hidden");
}

//Clears all input values to make then empty
function clearVal() {
    $("input").val("");
}


//loads all the commands so they can be editted or deleted via ajax
function getCommands() {

    $.ajax({
        url: 'commands.json',
        dataType: 'json',
        type: 'get',
        cache: false,
        success: function (data) {
            $(data.fixedCmd).each(function (index, value) {
                //console.log(value.desc);
                //console.log(value.example);
                //console.log(index);
                $('#json-commands').append(
                    `<div class="form-wrap">
                    <form class="form-signin" autocomplete="off" method="post">
                        <div class="form-label-group">
                            <input type="text" id="desc-${index}" class="form-control" value="${value.desc}"
                                readonly>
                            <label for="desc-${index}" readonly>Description</label>
                        </div>
                        <div class="form-label-group">
                            <input type="text" id="exp-${index}" class="form-control" value="${value.example}" readonly>
                            <label for="exp-${index}" readonly>Example</label>
                        </div>
                        <div class="row mt-4">
                            <button class="btn btn-lg btn-primary btn-block mb-2 mr-4 ml-4 col" name="edit" value="${index}">Edit</button>
                            <button class="btn btn-lg btn-primary btn-block mb-2 ml-4 mr-4 col" name="delete" value="${index}">Delete</button>
                        </div>
                    </form>
                </div>`
                );
            });
        }
    });
}

//loads all the help commands using Ajax.
function getHelpCommands() {

    $.ajax({
        url: 'commands.json',
        dataType: 'json',
        type: 'get',
        cache: false,
        success: function (data) {
            $(data.fixedCmd).each(function (index, value) {
                //console.log(value.desc);
                //console.log(value.example);
                //console.log(index);
                $('#json-commands').append(
                    `<div class="wrap">
                        <div class="row">
                            <div class="col-2 col-md-2 mx-auto">
                                <h1>#${index + 1}</h1>
                            </div>
                            <div class="col-10 col-md-10 mx-auto">
                                <form class="form-signin">
                                    <h4>${value.example}</h4>
                                    <p>${value.desc}</p>
                                </form>
                            </div>
                        </div>
                    </div>
                    <hr>`
                );
            });
        }
    });

}